/*	Filename: hooks.cc
	Hook handlers for mod_log_dbd
	
	Created	20061104	Brandon Fosdick <bfoz@bfoz.net>
	Copyright Brandon Fosdick 2006 (BSD License)
*/

#include "hooks.h"
#include "serverconfig.h"

//Apache includes
//	***	These must be local includes to resolve alloc conflict
#include "http_config.h"
#include "http_protocol.h"

namespace log_dbd
{
	//Post-config phase hook
	static int post_config(apr_pool_t* p, apr_pool_t* plog, apr_pool_t* ptemp, server_rec* s)
	{
		void *data = NULL;
		const char *userdata_key = "mod_log_dbd_post_config";
		apr_pool_userdata_get(&data, userdata_key, s->process->pool);
		if (data == NULL)
		{
			/* WARNING: This must *not* be apr_pool_userdata_setn(). */
			apr_pool_userdata_set((const void *)1, userdata_key,
			apr_pool_cleanup_null, s->process->pool);
			return OK; /* This would be the first time through */
		}

		while(s != NULL)
		{
			ServerConfig* cfg = get_server_config(s);	//Get the server config struct
			cfg->post_config();
//			if( cfg->post_config() )
//				CLOG(PTR_CHAR(s->server_hostname) << " is ready!");
			s = s->next;
		}
		return OK;
	}
	
	//Handle child init
	//	***	The server_rec list isn't walked by core, so it must be walked here
	static void child_init(apr_pool_t* p, server_rec* s)
	{
		TRACE();

		//Loop through all of the server records and initialize each
		while(s != NULL)
		{
			get_server_config(s)->connect();				//Open the DB connection
			s = s->next;
		}
	}

	static int log_transaction(request_rec* r)
	{
		return get_server_config(r)->log_transaction(r);
	}

	//Register handlers, hooks, etc
	void register_hooks(apr_pool_t* p)
	{
		TRACE();
		//Apache init hooks
		ap_hook_post_config(post_config, NULL, NULL, APR_HOOK_MIDDLE);
		ap_hook_child_init(child_init, NULL, NULL, APR_HOOK_MIDDLE);
		apr_dbd_init(p);		//Init the DBD library
		ap_hook_log_transaction(log_transaction, NULL, NULL, APR_HOOK_MIDDLE);

	}
};
