/*	Filename:	trace.h
*/

#include <stdio.h>
//#include <stdlib.h>
//#include <time.h>
//#include <stdarg.h>
#include <sys/stat.h>
#include <sstream>
#include <string>

#ifndef	TRACE_H
#define	TRACE_H

class	Trace
{
public:
	typedef	std::ostringstream	os_t;
	std::ostringstream	log_stream;

	explicit Trace(const char* mn, const char* fn, int ln) : module_name(mn), function_name(fn), line_number(ln)
	{
		open();

		if( fp==NULL )
			perror("trace constructor: ");
		else
		{
			depth_leader();
			fprintf(fp, "> %d %s:%d %s\n", depth, module_name, line_number, function_name);
		}
		++depth;
	}

	explicit Trace(const char* mn, const char* fn, int ln, const char* s) : module_name(mn), function_name(fn), line_number(ln)
	{
		open();

		if( fp==NULL )
			perror("trace constructor: ");
		else
		{
			depth_leader();
			fprintf(fp, "> %d %s:%d %s ( %s )\n", depth, module_name, line_number, function_name, s);
		}
		++depth;
	}

	~Trace()
	{
		--depth;

		if(fp==NULL)
			perror("trace destructor: ");
		else
		{
			depth_leader();
			fprintf(fp, "< %d %s:%d %s\n", depth, module_name, line_number, function_name);
			fflush(fp);
			if( depth == 0 )
			{
				fclose(fp);
				fp=NULL;
			}
		}
	}

	void flush(int ln)
	{
		if( fp != NULL )
		{
			for(int i=1; i<depth; ++i)	//Depth has already been incremented
				fputc('|', fp);
			fprintf(fp, "-- %d %s\n", ln, log_stream.str().c_str());
			log_stream.str(std::string());
		}
	}

private:
	static FILE*	fp;
	unsigned	line_number;
	const char *function_name;
	const char *module_name;
	static std::string	file_name;	//Trace file name
	static int	depth;	//Depth counter
	static bool	append;	//Should the file be created or appended?
	
	void open()
	{
		if( fp == NULL )
		{
			if( append )
				fp = fopen(file_name.c_str(), "a");
			else	//Clear the file on every restart
			{
				fp = fopen(file_name.c_str(), "w");
				chmod(file_name.c_str(), S_IRWXU | S_IRWXG | S_IRWXO);
				append = true;
			}
		}
	}
	
	void depth_leader()
	{
		for(int i=0; i<depth; ++i)
			fputc('|', fp);
	}
};

//Convenience macros
#ifdef WITH_TRACE
#	define	TRACE()	Trace trace(__FILE__, __func__, __LINE__);
#	define	TRACE1(a)	std::ostringstream osst; osst << a; Trace trace(__FILE__, __func__, __LINE__, osst.str().c_str());
#	define	TRACE_LOG(a)	trace.log_stream << a; trace.flush(__LINE__);
#	define	TRACE_VAR(a)	TRACE_LOG(#a " = " << (a));
#else
#	define	TRACE()
#	define	TRACE1(a)	TRACE()
#	define	TRACE_LOG(a)
#endif

#endif	//	TRACE_H
